==> v1.1.0.tar.gz: netaddr/netaddr.go <==
package netaddr

import (
	"bytes"
	"fmt"
	"net"
)

type NetAddr struct {
	Address []byte
	Length  byte
}

func applyMask(address, mask []byte) []byte {
	var ip net.IP = net.IP(address)

	return ip.Mask(mask)
}

func (na *NetAddr) AddressSize() int {
	return len(na.Address)
}

func (na *NetAddr) FirstAddress() *NetAddr {
	bitSize := na.AddressSize() * 8
	length := na.Length
	mask := net.CIDRMask(int(length), bitSize)
	addressableSlice := na.Address
	first0 := applyMask(addressableSlice, mask)
	first := make([]byte, na.AddressSize())

	copy(first[:], first0)

	return &NetAddr{Address: first, Length: length}
}

func (na *NetAddr) LastAddress() *NetAddr {
	bitSize := na.AddressSize() * 8
	length := na.Length
	mask := net.CIDRMask(int(length), bitSize)
	allOnes := net.CIDRMask(bitSize, bitSize)
	invMask := byteXor(mask, allOnes)
	lastAddr := byteOr(na.FirstAddress().Address, invMask)

	return &NetAddr{Address: lastAddr, Length: length}
}

func byteOp(u, v []byte, f func(byte, byte) byte) []byte {
	w := make([]byte, len(u), len(u))

	for i := 0; i < len(u); i++ {
		w[i] = f(u[i], v[i])
	}
	return w
}

func byteOr(u, v []byte) []byte {
	return byteOp(u, v, func(ui, vi byte) byte {
		return ui | vi
	})
}

func byteXor(u, v []byte) []byte {
	return byteOp(u, v, func(ui, vi byte) byte {
		return ui ^ vi
	})
}

func SetLength(na *NetAddr, length byte) *NetAddr {
	return &NetAddr{
		Address: na.Address,
		Length:  length % byte(na.AddressSize()*8),
	}
}

func IsEqual(na1, na2 *NetAddr) bool {
	return na1.Length == na2.Length &&
		bytes.Equal(na1.Address, na2.Address)
}

func Contains(na1, na2 *NetAddr) bool {
	tmp := SetLength(na2, na1.Length)

	return na1.Length <= na2.Length &&
		IsEqual(na1.FirstAddress(), tmp.FirstAddress())
}

func IP(s string) (*NetAddr, error) {
	var ip net.IP
	var ipNet *net.IPNet
	var err error

	ip, ipNet, err = net.ParseCIDR(s)
	if err != nil {
		return &NetAddr{}, err
	}

	var length, _ int = net.IPMask.Size(ipNet.Mask)
	var address []byte = ip.To4()

	if address == nil {
		address = ip.To16()
	}
	if address == nil {
		panic(fmt.Sprintf("unexpected failure parsing '%s'\n", s))
	}

	na := NetAddr{Address: address, Length: byte(length)}

	return &na, nil
}

==> v1.2.0.tar.gz: netaddr/netaddr.go <==
package netaddr

import (
	"bytes"
	"fmt"
	"net"
)

type NetAddr struct {
	Address []byte
	Length  byte
}

func applyMask(address, mask []byte) []byte {
	var ip net.IP = net.IP(address)

	return ip.Mask(mask)
}

func (na *NetAddr) AddressSize() int {
	return len(na.Address)
}

func (na *NetAddr) FirstAddress() *NetAddr {
	bitSize := na.AddressSize() * 8
	length := na.Length
	mask := net.CIDRMask(int(length), bitSize)
	addressableSlice := na.Address
	first0 := applyMask(addressableSlice, mask)
	first := make([]byte, na.AddressSize())

	copy(first[:], first0)

	return &NetAddr{Address: first, Length: length}
}

func (na *NetAddr) LastAddress() *NetAddr {
	bitSize := na.AddressSize() * 8
	length := na.Length
	mask := net.CIDRMask(int(length), bitSize)
	allOnes := net.CIDRMask(bitSize, bitSize)
	invMask := byteXor(mask, allOnes)
	lastAddr := byteOr(na.FirstAddress().Address, invMask)

	return &NetAddr{Address: lastAddr, Length: length}
}

func (na *NetAddr) String() string {
	addr := net.IP(na.Address).String()
	length := fmt.Sprintf("%d", na.Length)
	return addr + "/" + length
}

func (na *NetAddr) IsRFC1918() bool {
	return IsSubset(na, &NetAddr{
		Address: []byte{10, 0, 0, 0},
		Length:  byte(8),
	}) || IsSubset(na, &NetAddr{
		Address: []byte{172, 16, 0, 0},
		Length:  byte(12),
	}) || IsSubset(na, &NetAddr{
		Address: []byte{192, 168, 0, 0},
		Length:  byte(16),
	})
}

func byteOp(u, v []byte, f func(byte, byte) byte) []byte {
	w := make([]byte, len(u), len(u))

	for i := 0; i < len(u); i++ {
		w[i] = f(u[i], v[i])
	}
	return w
}

func byteOr(u, v []byte) []byte {
	return byteOp(u, v, func(ui, vi byte) byte {
		return ui | vi
	})
}

func byteXor(u, v []byte) []byte {
	return byteOp(u, v, func(ui, vi byte) byte {
		return ui ^ vi
	})
}

func SetLength(na *NetAddr, length byte) *NetAddr {
	return &NetAddr{
		Address: na.Address,
		Length:  length % byte(na.AddressSize()*8),
	}
}

func IsEqual(na1, na2 *NetAddr) bool {
	return na1.Length == na2.Length &&
		bytes.Equal(na1.Address, na2.Address)
}

func Contains(na1, na2 *NetAddr) bool {
	tmp := SetLength(na2, na1.Length)

	return na1.Length <= na2.Length &&
		IsEqual(na1.FirstAddress(), tmp.FirstAddress())
}

func IsSubset(na1, na2 *NetAddr) bool {
	tmp := SetLength(na1, na2.Length)

	return na2.Length <= na1.Length &&
		IsEqual(na2.FirstAddress(), tmp.FirstAddress())
}

func IP(s string) (na *NetAddr, err error) {
	var ip net.IP
	var ipNet *net.IPNet

	ip, ipNet, err = net.ParseCIDR(s)
	switch err.(type) {
	case nil:
	case *net.ParseError:
		ip = net.ParseIP(s)
		if ip == nil {
			err = fmt.Errorf("invalid ip address: %v", s)
			return
		}
		//var mask []byte
		if test := ip.To4(); test != nil {
			ipNet = &net.IPNet{
				IP:   ip,
				Mask: net.IPMask([]byte{255, 255, 255, 255}),
			}
		} else if test := ip.To16(); test != nil {
			ipNet = &net.IPNet{
				IP: ip,
				Mask: net.IPMask(
					[]byte{
						0xff, 0xff, 0xff, 0xff,
						0xff, 0xff, 0xff, 0xff,
						0xff, 0xff, 0xff, 0xff,
						0xff, 0xff, 0xff, 0xff,
					},
				),
			}
		}
		err = nil
	default:
		return
	}
	var length, _ int = net.IPMask.Size(ipNet.Mask)
	var address []byte = ip.To4()

	if address == nil {
		address = ip.To16()
	}
	if address == nil {
		panic(fmt.Sprintf("unexpected failure parsing '%s'\n", s))
	}
	na = &NetAddr{Address: address, Length: byte(length)}
	return
}
